package top.zhongmingmao.blog.clientapp.controller;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.experimental.FieldDefaults;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.ModelAndView;
import top.zhongmingmao.blog.clientapp.oauth2.AuthorizationCodeTokenService;
import top.zhongmingmao.blog.clientapp.oauth2.OAuth2Token;
import top.zhongmingmao.blog.clientapp.oauth2.UserInfo;
import top.zhongmingmao.blog.clientapp.security.ClientUserDetails;
import top.zhongmingmao.blog.clientapp.user.ClientUser;
import top.zhongmingmao.blog.clientapp.user.ClientUserRepository;

import java.net.URI;
import java.util.Calendar;
import java.util.Date;
import java.util.Objects;

import static top.zhongmingmao.blog.clientapp.oauth2.AuthorizationCodeFlowUtils.ENDPOINT_USER;
import static top.zhongmingmao.blog.clientapp.oauth2.AuthorizationCodeFlowUtils.buildAuthorizeEndpoint;

@Controller
@AllArgsConstructor
@FieldDefaults(level = AccessLevel.PRIVATE)
public class MainPage {

  ClientUserRepository userRepository;
  AuthorizationCodeTokenService tokenService;

  @GetMapping("/")
  public String home() {
    return "index";
  }

  @GetMapping("/mainpage")
  public ModelAndView mainpage() {
    ClientUserDetails userDetails =
        (ClientUserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
    ClientUser clientUser = userDetails.getDelegator();

    if (Objects.isNull(clientUser.getAccessToken())) {
      return new ModelAndView("redirect:" + buildAuthorizeEndpoint());
    }

    ModelAndView mv = new ModelAndView("mainpage");
    mv.addObject("user", clientUser);
    fetchUserInfo(mv, clientUser.getAccessToken());
    return mv;
  }

  @GetMapping("/callback")
  public ModelAndView callback(final String code, final String state) {
    ClientUserDetails userDetails =
        (ClientUserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
    ClientUser clientUser = userDetails.getDelegator();

    OAuth2Token token = tokenService.exchange(code);
    clientUser.setAccessToken(token.getAccessToken());

    Calendar tokenValidity = Calendar.getInstance();
    long validIn = System.currentTimeMillis() + Long.parseLong(token.getExpiresIn());
    tokenValidity.setTime(new Date(validIn));
    clientUser.setAccessTokenValidity(tokenValidity);

    userRepository.save(clientUser);

    return new ModelAndView("redirect:/mainpage");
  }

  private void fetchUserInfo(final ModelAndView mv, final String token) {
    RestTemplate rest = new RestTemplate();
    MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
    headers.add("Authorization", "Bearer " + token);
    RequestEntity<Object> request =
        new RequestEntity<>(headers, HttpMethod.GET, URI.create(ENDPOINT_USER));
    ResponseEntity<UserInfo> response = rest.exchange(request, UserInfo.class);
    if (response.getStatusCode().is2xxSuccessful()) {
      mv.addObject("userInfo", response.getBody());
    }
  }
}
