package top.zhongmingmao.blog.clientapp.oauth2;

import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.nio.charset.StandardCharsets;
import java.util.*;

@Slf4j
public final class AuthorizationCodeFlowUtils {

  public static final String HOST = "http://localhost:8000";
  public static final String ENDPOINT_AUTHORIZE = HOST + "/oauth/authorize";
  public static final String ENDPOINT_TOKEN = HOST + "/oauth/token";
  public static final String ENDPOINT_USER = HOST + "/api/userInfo";
  public static final String ENDPOINT_CALLBACK = "http://localhost:9000/callback";

  public static String buildAuthorizeEndpoint() {
    Map<String, String> params = new HashMap<>(1 << 2);
    params.put("client_id", "wechat");
    params.put("redirect_uri", ENDPOINT_CALLBACK);
    params.put("response_type", "code");
    params.put("scope", "read_userinfo");

    List<String> paramList = new ArrayList<>(params.size());
    params.forEach((name, value) -> paramList.add(String.join("=", name, value)));
    String endpoint =
        String.format(
            "%s?%s",
            ENDPOINT_AUTHORIZE,
            paramList.stream().reduce((a, b) -> String.join("&", a, b)).orElse(""));
    log.info("authorize endpoint: {}", endpoint);
    return endpoint;
  }

  public static String encodeClientCredentials(final String clientId, final String clientSecret) {
    String clientCredentials =
        new String(
            Base64.getEncoder()
                .encode(String.join(":", clientId, clientSecret).getBytes(StandardCharsets.UTF_8)));
    log.info(
        "clientId: {}, clientSecret: {}, clientCredentials: {}",
        clientId,
        clientSecret,
        clientCredentials);
    return clientCredentials;
  }

  public static HttpHeaders buildExchangeCodeHeader(final String encodedClientCredentials) {
    HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
    headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON_UTF8));
    headers.add("Authorization", "Basic " + encodedClientCredentials);
    return headers;
  }

  public static MultiValueMap<String, String> buildExchangeCodeBody(final String code) {
    MultiValueMap<String, String> form = new LinkedMultiValueMap<>();
    form.add("redirect_uri", ENDPOINT_CALLBACK);
    form.add("grant_type", "authorization_code");
    form.add("scope", "read_userinfo");
    form.add("code", code);
    return form;
  }
}
